@@ -204,8 +204,8 @@ static void
 gail_button_real_initialize (AtkObject *obj,
                              gpointer   data)
 {
-  GailContainer *container = GAIL_CONTAINER (obj);
   GailButton *button = GAIL_BUTTON (obj);
+  guint handler_id;
 
   ATK_OBJECT_CLASS (parent_class)->initialize (obj, data);
 
@@ -213,8 +213,10 @@ gail_button_real_initialize (AtkObject *obj,
    * As we report the button as having no children we are not interested
    * in add and remove signals
    */
-  g_signal_handler_disconnect (data, container->add_handler);
-  g_signal_handler_disconnect (data, container->remove_handler);
+  handler_id = GPOINTER_TO_UINT (g_object_get_data (G_OBJECT (obj), "gail-add-handler-id"));
+  g_signal_handler_disconnect (data, handler_id);
+  handler_id = GPOINTER_TO_UINT (g_object_get_data (G_OBJECT(obj), "gail-remove-handler-id"));
+  g_signal_handler_disconnect (data, handler_id);
 
   button->state = GTK_STATE_NORMAL;
 
