@@ -27,10 +27,10 @@ static void gail_entry_factory_class_init (GailEntryFactoryClass       *klass);
 static AtkObject* gail_entry_factory_create_accessible (
                                GObject                              *obj);
 
-static AtkObjectFactoryClass    *parent_class = NULL;
+static GType gail_entry_factory_get_accessible_type (void);
 
 GType
-gail_entry_factory_get_type ()
+gail_entry_factory_get_type (void)
 {
   static GType type = 0;
 
@@ -61,20 +61,10 @@ gail_entry_factory_class_init (GailEntryFactoryClass *klass)
 {
   AtkObjectFactoryClass *class = ATK_OBJECT_FACTORY_CLASS (klass);
 
-  parent_class = g_type_class_ref (ATK_TYPE_OBJECT_FACTORY);
-
   class->create_accessible = gail_entry_factory_create_accessible;
+  class->get_accessible_type = gail_entry_factory_get_accessible_type;
 }
 
-AtkObjectFactory* gail_entry_factory_new ()
-{
-  GObject *factory;
-
-  factory = g_object_new (GAIL_TYPE_ENTRY_FACTORY, NULL);
-
-  return ATK_OBJECT_FACTORY (factory);
-} 
-
 static AtkObject* 
 gail_entry_factory_create_accessible (GObject     *obj)
 {
@@ -86,3 +76,9 @@ gail_entry_factory_create_accessible (GObject     *obj)
 
   return gail_entry_new (widget);
 }
+
+static GType
+gail_entry_factory_get_accessible_type (void)
+{
+  return GAIL_TYPE_ENTRY;
+}
