@@ -17,15 +17,15 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#include <string.h>
 #include <gtk/gtk.h>
-#include <stdlib.h>
 #include "gaillabel.h"
 #include "gaillabelfactory.h"
-#include "gailtexthelper.h"
+#include <util/gailmisc.h>
 
 static void       gail_label_class_init            (GailLabelClass    *klass);
-static void	  gail_label_widget_init	   (GailWidget	      *widget,
-                                                    GtkWidget	      *gtk_widget);
+static void	  gail_label_real_initialize	   (AtkObject 	      *obj,
+                                                    gpointer	      data);
 static void	  gail_label_real_notify_gtk	   (GObject	      *obj,
                                                     GParamSpec	      *pspec);
 static void       gail_label_finalize              (GObject           *object);
@@ -146,37 +146,36 @@ gail_label_class_init (GailLabelClass *klass)
   gobject_class->finalize = gail_label_finalize;
 
   widget_class = (GailWidgetClass*)klass;
-  widget_class->init = gail_label_widget_init;
   widget_class->notify_gtk = gail_label_real_notify_gtk;
 
-  parent_class = g_type_class_ref (GAIL_TYPE_WIDGET);
+  parent_class = g_type_class_peek_parent (klass);
 
   class->get_name = gail_label_get_name;
   class->ref_state_set = gail_label_ref_state_set;
   class->ref_relation_set = gail_label_ref_relation_set;
-
+  class->initialize = gail_label_real_initialize;
 }
 
 static void
-gail_label_widget_init (GailWidget *widget,
-                        GtkWidget  *gtk_widget)
+gail_label_real_initialize (AtkObject *obj,
+                            gpointer  data)
 {
   GtkLabel  *label;
   GailLabel *gail_label;
   const gchar *label_text;
 
-  parent_class->init (widget, gtk_widget);
+  ATK_OBJECT_CLASS (parent_class)->initialize (obj, data);
   
-  gail_label = GAIL_LABEL (widget);
+  gail_label = GAIL_LABEL (obj);
 
   gail_label->cursor_position = -1;
   gail_label->selection_bound = -1;
   
-  gail_label->texthelper = gail_text_helper_new ();
+  gail_label->textutil = gail_text_util_new ();
 
-  label = GTK_LABEL (gtk_widget);
+  label = GTK_LABEL (data);
   label_text = gtk_label_get_text (label);
-  gail_text_helper_text_setup (gail_label->texthelper, label_text);
+  gail_text_util_text_setup (gail_label->textutil, label_text);
   
   if (label_text == NULL)
     gail_label->label_length = 0;
@@ -184,25 +183,23 @@ gail_label_widget_init (GailWidget *widget,
     gail_label->label_length = g_utf8_strlen (label_text, -1);
 }
 
-GtkAccessible* 
+AtkObject* 
 gail_label_new (GtkWidget *widget)
 {
   GObject *object;
-  GtkAccessible *accessible;
+  AtkObject *accessible;
 
   g_return_val_if_fail (GTK_IS_LABEL (widget), NULL);
 
   object = g_object_new (GAIL_TYPE_LABEL, NULL);
 
-  g_return_val_if_fail (GTK_IS_ACCESSIBLE (object), NULL);
+  accessible = ATK_OBJECT (object);
+  atk_object_initialize (accessible, widget);
 
-  gail_widget_init (GAIL_WIDGET (object), widget);
-
-  accessible = GTK_ACCESSIBLE (object);
   if (GTK_IS_ACCEL_LABEL (widget))
-    ATK_OBJECT(accessible)->role = ATK_ROLE_ACCEL_LABEL;
+    accessible->role = ATK_ROLE_ACCEL_LABEL;
   else
-    ATK_OBJECT(accessible)->role = ATK_ROLE_LABEL;
+    accessible->role = ATK_ROLE_LABEL;
 
   return accessible;
 }
@@ -229,11 +226,9 @@ gail_label_real_notify_gtk (GObject           *obj,
  
       label_text = gtk_label_get_text (label);
 
-      gail_text_helper_text_setup (gail_label->texthelper, "");
-      g_signal_emit_by_name (obj, "text_changed::delete", 0, 
+      g_signal_emit_by_name (gail_obj, "text_changed::delete", 0, 
                              gail_label->label_length);
-      gail_label->label_length = g_utf8_strlen (gtk_label_get_text (label), -1);
-      gail_text_helper_text_setup (gail_label->texthelper, label_text);
+      gail_text_util_text_setup (gail_label->textutil, label_text);
       g_signal_emit_by_name (gail_obj, "text_changed::insert", 0, 
                              gail_label->label_length);
 
@@ -294,7 +289,7 @@ gail_label_real_notify_gtk (GObject           *obj,
           g_signal_emit_by_name (gail_obj, "text_caret_moved", 
                                  gail_label->cursor_position);
         if (selection_changed)
-          g_signal_emit_by_name (gail_obj, "selection_changed");
+          g_signal_emit_by_name (gail_obj, "text_selection_changed");
 
     }
   else
@@ -306,7 +301,7 @@ gail_label_finalize (GObject            *object)
 {
   GailLabel *label = GAIL_LABEL (object);
 
-  g_object_unref (label->texthelper);
+  g_object_unref (label->textutil);
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
@@ -388,7 +383,6 @@ gail_label_get_name (AtkObject *accessible)
        * Get the text on the label
        */
       GtkWidget *widget;
-      GValue value = { 0, };
 
       widget = GTK_ACCESSIBLE (accessible)->widget;
       if (widget == NULL)
@@ -399,9 +393,7 @@ gail_label_get_name (AtkObject *accessible)
 
       g_return_val_if_fail (GTK_IS_LABEL (widget), NULL);
 
-      g_value_init (&value, G_TYPE_STRING);
-      g_object_get_property (G_OBJECT (widget), "label", &value);
-      return g_value_get_string (&value);
+      return gtk_label_get_text (GTK_LABEL (widget));
     }
 }
 
@@ -453,8 +445,8 @@ gail_label_get_text (AtkText *text,
     return NULL;
   else
   {
-    return gail_text_helper_get_substring (GAIL_LABEL(text)->texthelper, 
-                                           &start_pos, &end_pos);
+    return gail_text_util_get_substring (GAIL_LABEL(text)->textutil, 
+                                         &start_pos, &end_pos);
   }
 }
 
@@ -477,7 +469,7 @@ gail_label_get_text_before_offset (AtkText         *text,
   /* Get label */
   label = GTK_LABEL (widget);
 
-  return gail_text_helper_get_text (GAIL_LABEL (text)->texthelper,
+  return gail_text_util_get_text (GAIL_LABEL (text)->textutil,
                            gtk_label_get_layout (label), GAIL_BEFORE_OFFSET, 
                            boundary_type, offset, start_offset, end_offset); 
 }
@@ -501,7 +493,7 @@ gail_label_get_text_at_offset (AtkText         *text,
   /* Get label */
   label = GTK_LABEL (widget);
 
-  return gail_text_helper_get_text (GAIL_LABEL (text)->texthelper,
+  return gail_text_util_get_text (GAIL_LABEL (text)->textutil,
                               gtk_label_get_layout (label), GAIL_AT_OFFSET, 
                               boundary_type, offset, start_offset, end_offset);
 }
@@ -527,7 +519,7 @@ gail_label_get_text_after_offset (AtkText         *text,
   /* Get label */
   label = GTK_LABEL (widget);
 
-  return gail_text_helper_get_text (GAIL_LABEL (text)->texthelper,
+  return gail_text_util_get_text (GAIL_LABEL (text)->textutil,
                            gtk_label_get_layout (label), GAIL_AFTER_OFFSET, 
                            boundary_type, offset, start_offset, end_offset);
 }
@@ -592,7 +584,7 @@ gail_label_get_n_selections (AtkText *text)
 
   label = GTK_LABEL (widget);
 
-  if (gtk_label_get_selectable (label) == FALSE)
+  if (!gtk_label_get_selectable (label))
      return 0;
 
   if (gtk_label_get_selection_bounds (label, &start, &end))
@@ -620,7 +612,7 @@ gail_label_get_selection (AtkText *text,
  /* Only let the user get the selection if one is set, and if the
   * selection_num is 0.
   */
-  if (gtk_label_get_selectable( label) == FALSE || selection_num != 0)
+  if (!gtk_label_get_selectable( label) || selection_num != 0)
      return NULL;
 
   if (gtk_label_get_selection_bounds (label, start_pos, end_pos))
@@ -630,7 +622,7 @@ gail_label_get_selection (AtkText *text,
       if (label_text == NULL)
         return 0;
       else
-        return gail_text_helper_get_substring (GAIL_LABEL (text)->texthelper, 
+        return gail_text_util_get_substring (GAIL_LABEL (text)->textutil, 
                                              start_pos, end_pos);
     }
   else 
@@ -653,7 +645,7 @@ gail_label_add_selection (AtkText *text,
 
   label = GTK_LABEL (widget);
 
-  if (gtk_label_get_selectable (label) == FALSE)
+  if (!gtk_label_get_selectable (label))
      return FALSE;
 
   if (! gtk_label_get_selection_bounds (label, &start, &end))
@@ -683,7 +675,7 @@ gail_label_remove_selection (AtkText *text,
 
   label = GTK_LABEL (widget);
 
-  if (gtk_label_get_selectable (label) == FALSE)
+  if (!gtk_label_get_selectable (label))
      return FALSE;
 
   if (gtk_label_get_selection_bounds (label, &start, &end))
@@ -715,7 +707,7 @@ gail_label_set_selection (AtkText *text,
 
   label = GTK_LABEL (widget);
 
-  if (gtk_label_get_selectable (label) == FALSE)
+  if (!gtk_label_get_selectable (label))
      return FALSE;
 
   if (gtk_label_get_selection_bounds (label, &start, &end))
@@ -753,7 +745,7 @@ gail_label_get_character_extents (AtkText      *text,
   index = g_utf8_offset_to_pointer (label->text, offset) - label->text;
   pango_layout_index_to_pos (gtk_label_get_layout (label), index, &char_rect);
   
-  gail_text_helper_get_extents_from_pango_rectangle (widget, &char_rect, 
+  gail_misc_get_extents_from_pango_rectangle (widget, &char_rect, 
                     x_layout, y_layout, x, y, width, height, coords);
 } 
 
@@ -775,7 +767,7 @@ gail_label_get_offset_at_point (AtkText      *text,
   
   gtk_label_get_layout_offsets (label, &x_layout, &y_layout);
   
-  index = gail_text_helper_get_index_at_point_in_layout (widget, 
+  index = gail_misc_get_index_at_point_in_layout (widget, 
                                               gtk_label_get_layout (label), 
                                               x_layout, y_layout, x, y, coords);
   if (index == -1)
@@ -807,19 +799,19 @@ gail_label_get_run_attributes (AtkText        *text,
   justify = gtk_label_get_justify (label);
   if (justify != GTK_JUSTIFY_CENTER)
     {
-      at_set = gail_text_helper_add_attribute (at_set, 
-                                               ATK_TEXT_ATTR_JUSTIFICATION,
-     g_strdup (atk_attribute_get_value (ATK_TEXT_ATTR_JUSTIFICATION, justify)));
+      at_set = gail_misc_add_attribute (at_set, 
+                                        ATK_TEXT_ATTR_JUSTIFICATION,
+     g_strdup (atk_text_attribute_get_value (ATK_TEXT_ATTR_JUSTIFICATION, justify)));
     }
   dir = gtk_widget_get_direction (widget);
   if (dir == GTK_TEXT_DIR_RTL)
     {
-      at_set = gail_text_helper_add_attribute (at_set, 
-                                               ATK_TEXT_ATTR_DIRECTION,
-     g_strdup (atk_attribute_get_value (ATK_TEXT_ATTR_DIRECTION, dir)));
+      at_set = gail_misc_add_attribute (at_set, 
+                                        ATK_TEXT_ATTR_DIRECTION,
+     g_strdup (atk_text_attribute_get_value (ATK_TEXT_ATTR_DIRECTION, dir)));
     }
 
-  at_set = gail_text_helper_get_run_attributes (at_set,
+  at_set = gail_misc_layout_get_run_attributes (at_set,
                                                 gtk_label_get_layout (label),
                                                 label->text,
                                                 offset,
@@ -843,18 +835,9 @@ gail_label_get_default_attributes (AtkText        *text)
 
   label = GTK_LABEL (widget);
   
-  at_set = gail_text_helper_add_attribute (at_set, 
-                                           ATK_TEXT_ATTR_JUSTIFICATION, 
-     g_strdup (atk_attribute_get_value (ATK_TEXT_ATTR_JUSTIFICATION, 
-                                        gtk_label_get_justify (label))));
-  wrap_value = gtk_label_get_line_wrap (label);
-  at_set = gail_text_helper_add_attribute (at_set,
-                                           ATK_TEXT_ATTR_WRAP_MODE,
-              g_strdup (atk_attribute_get_value (ATK_TEXT_ATTR_WRAP_MODE, 
-                                                 wrap_value  ? 0 : 2))); 
-  at_set = gail_text_helper_get_default_attributes (at_set,
-                                                 gtk_label_get_layout (label),
-                                                    widget);
+  at_set = gail_misc_get_default_attributes (at_set,
+                                             gtk_label_get_layout (label),
+                                             widget);
   return at_set;
 }
 
