@@ -30,7 +30,7 @@ static AtkObject* gail_pixmap_factory_create_accessible (
 static GType gail_pixmap_factory_get_accessible_type (void);
 
 GType
-gail_pixmap_factory_get_type ()
+gail_pixmap_factory_get_type (void)
 {
   static GType type = 0;
 
@@ -65,16 +65,6 @@ gail_pixmap_factory_class_init (GailPixmapFactoryClass *klass)
   class->get_accessible_type = gail_pixmap_factory_get_accessible_type;
 }
 
-AtkObjectFactory*
-gail_pixmap_factory_new ()
-{
-  GObject *factory;
-
-  factory = g_object_new (GAIL_TYPE_PIXMAP_FACTORY, NULL);
-
-  return ATK_OBJECT_FACTORY (factory);
-} 
-
 static AtkObject* 
 gail_pixmap_factory_create_accessible (GObject     *obj)
 {
